/*
 * Copyright 2018-2019 ON Semiconductor
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __FUSB251_INC__
#define __FUSB251_INC__

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/list.h>
#include <linux/semaphore.h>
#include <linux/workqueue.h>
#include <linux/switch.h>

/* Register map for the FUSB251 */
#define FUSB251_PRODUCT_ID      0x01
#define FUSB251_CONTROL         0x02
#define FUSB251_INTERRUPT       0x03
#define FUSB251_INT_MASK        0x04
#define FUSB251_STATUS          0x05
#define FUSB251_MOISTATUS       0x06
#define FUSB251_SWITCHCONTROL   0x07
#define FUSB251_THRESHOLD1      0x08
#define FUSB251_THRESHOLD2      0x09
#define FUSB251_TIMER1          0x0A
#define FUSB251_RESET           0x0B
#define FUSB251_TIMER2          0x0C
#define FUSB251_REG_NUM         12

/* for interrupt */
#define MSK_I_OVP               1
#define MSK_I_OVP_REC           (1 << 1)
#define MSK_I_MOS_DET           (1 << 2)
#define MSK_I_DRY_DET           (1 << 3)
#define MSK_I_CC1_TIMER         (1 << 4)
#define MSK_I_CC2_TIMER         (1 << 5)
#define MSK_I_MASK_ALL          0x3F
/* for control */
#define CTL_EN_CC_DET           (1 << 1)
#define CTL_EN_SBU_DET          (1 << 4)
#define CTL_EN_DRY_DET          (1 << 5)
#define CTL_DISABLE_ALL         0x00
#define EN_CC_DET_MASK          0x01
#define EN_CC_DET_SHIFT         1
#define EN_SBU_DET_MASK         0x01
#define EN_SBU_DET_SHIFT        2
#define EN_AUTO_SBU_DET_MASK    0x01
#define EN_AUTO_SBU_DET_SHIFT   4
#define EN_DRY_DET_MASK         0x01
#define EN_DRY_DET_SHIFT        5
/* for status */
#define STATUS_LOOK4DRY_MASK    0x01
#define STATUS_LOOK4DRY_SHIFT   5
#define STATUS_LOOK4SBU_MASK    0x01
#define STATUS_LOOK4SBU_SHIFT   6
#define STATUS_LOOK4CC_MASK     0x01
#define STATUS_LOOK4CC_SHIFT    7
/* for moisture status */
#define MOISTATUS_CC1MOS_MASK   0x01
#define MOISTATUS_CC1MOS_SHIFT  0
#define MOISTATUS_CC2MOS_MASK   0x01
#define MOISTATUS_CC2MOS_SHIFT  1
#define MOISTATUS_SBU1MOS_MASK  0x01
#define MOISTATUS_SBU1MOS_SHIFT 2
#define MOISTATUS_SBU2MOS_MASK  0x01
#define MOISTATUS_SBU2MOS_SHIFT 3
#define MOISTATUS_SBU1FT_MASK   0x01
#define MOISTATUS_SBU1FT_SHIFT  4
#define MOISTATUS_SBU2FT_MASK   0x01
#define MOISTATUS_SBU2FT_SHIFT  5
/* for switch */
#define SWITCH_CC_MASK          0x01
#define SWITCH_CC_SHIFT         0
#define SWITCH_SBU_MASK         0x03
#define SWITCH_SBU_SHIFT        1
/* for reset */
#define RESET_DEVICE            0x01
#define RESET_MOISTURE          0x02
/* for timer1 */
#define TIMER_DRY_DET_1S        0x03
#define TIMER_DRY_DET_2S        0x04
#define TIMER_DRY_DET_4S        0x05
#define TIMER_DRY_DET_8S        0x06
#define TIMER_DRY_DET_10S       0x07
/* for timer2 */
#define ADC_READ_TIMES_1        0x00
#define ADC_READ_TIMES_2        0x04
#define ADC_READ_TIMES_3        0x08
#define CC_SETTLE_400US         0x00
#define CC_SETTLE_300US         0x01
#define CC_SETTLE_500US         0x02
#define CC_SETTLE_600US         0x03
/* for threshold1 */
#define SBU_MOS_R_DET_MASK      0x0F
#define SBU_MOS_R_DET_SHIFT     4
#define SBU_MOS_R_DET_480K      0xB0
#define SBU_MOS_R_DET_1280K     0xF0
#define CC_MOS_R_DET_MASK       0x0F
#define CC_MOS_R_DET_SHIFT      0
#define CC_MOS_R_DET_480K       0x0B
#define CC_MOS_R_DET_1280K      0x0F
/* for threshold2 */
#define SBU_FLOAT_DET_100MV     0x00
#define SBU_FLOAT_DET_200MV     0x10
#define SBU_FLOAT_DET_300MV     0x20
#define SBU_FLOAT_DET_400MV     0x30
#define SBU_FLOAT_DET_500MV     0x40
#define SBU_FLOAT_DET_600MV     0x50
#define SBU_FLOAT_DET_700MV     0x60
#define SBU_FLOAT_DET_800MV     0x70
#define VDRY_R_DET_MASK         0x0F
#define VDRY_R_DET_SHIFT        0
#define VDRY_DET_480K           0x0B
#define VDRY_DET_747K           0x0D
#define VDRY_DET_1280K          0x0F

/* Register unions */
typedef union {
	unsigned char byte;
	struct {
		unsigned char rev_id               : 2;
		unsigned char prd_id               : 2;
		unsigned char dev_id               : 4;
	};
} product_id_reg_t; /* R */

typedef union {
	unsigned char byte;
	struct {
		unsigned char man_sw               : 1;
		unsigned char en_cc_mos            : 1;
		unsigned char en_sbu_mos           : 1;
		unsigned char en_sbuft             : 1;
		unsigned char auto_en_sbu          : 1;
		unsigned char en_dry_check         : 1;
		unsigned char reserved             : 1;
		unsigned char dbg_acc_det          : 1;
	};
} control_reg_t; /* R/W */

typedef union {
	unsigned char byte;
	struct {
		unsigned char i_ovp                : 1;
		unsigned char i_ovp_rec            : 1;
		unsigned char i_mos_chg            : 1;
		unsigned char i_dry_chg            : 1;
		unsigned char i_cc1_timer          : 1;
		unsigned char i_cc2_timer          : 1;
		unsigned char reserved             : 2;
	};
} interrupt_reg_t; /* RC */

typedef union {
	unsigned char byte;
	struct {
		unsigned char m_ovp                : 1;
		unsigned char m_ovp_rec            : 1;
		unsigned char m_mos_det            : 1;
		unsigned char m_dry_det            : 1;
		unsigned char m_cc1_timer          : 1;
		unsigned char m_cc2_timer          : 1;
		unsigned char reserved             : 2;
	};
} mask_reg_t; /* R/W */

typedef union {
	unsigned char byte;
	struct {
		unsigned char ovp_cc               : 1;
		unsigned char ovp_sbu              : 1;
		unsigned char reserved             : 3;
		unsigned char look4dry             : 1;
		unsigned char look4sbu             : 1;
		unsigned char look4cc              : 1;
	};
} status_reg_t; /* R */

typedef union {
	unsigned char byte;
	struct {
		unsigned char cc1_mos              : 1;
		unsigned char cc2_mos              : 1;
		unsigned char sbu1_mos             : 1;
		unsigned char sbu2_mos             : 1;
		unsigned char sbu1_ft              : 1;
		unsigned char sbu2_ft              : 1;
		unsigned char fault                : 2;
	};
} moisture_status_t; /* R */

typedef union {
	unsigned char byte;
	struct {
		unsigned char cc_switch            : 1;
		unsigned char sbu_switch           : 2;
		unsigned char reserved             : 5;
	};
} switch_control_t; /* R/W */

typedef union {
	unsigned char byte;
	struct {
		unsigned char cc_mos_det           : 4;
		unsigned char sbu_mos_det          : 4;
	};
} threshold1_reg_t; /* R/W */

typedef union {
	unsigned char byte;
	struct {
		unsigned char vdry                 : 4;
		unsigned char sbu_float_det        : 3;
		unsigned char reserved             : 1;
	};
} threshold2_reg_t; /* R/W */

typedef union {
	unsigned char byte;
	struct {
		unsigned char tdry                 : 3;
		unsigned char reserved             : 5;
	};
} timer_reg_t; /* R/W */

typedef union {
	unsigned char byte;
	struct {
		unsigned char reset                : 1;
		unsigned char mos_reset            : 1;
		unsigned char reserved             : 6;
		/* [7..2] reserved */
	};
} reset_reg_t; /* R/W */

typedef union {
	unsigned char byte;
	struct {
		unsigned char cc_settle_time       : 2;
		unsigned char num_adc_read         : 2;
		unsigned char reserved             : 4;
	};
} timer2_reg_t; /* R/W */

typedef struct {
	product_id_reg_t              productid;
	control_reg_t                 control;
	interrupt_reg_t               interrupt;
	mask_reg_t                    mask;
	status_reg_t                  status;
	moisture_status_t             mos_status;
	switch_control_t              switch_control;
	threshold1_reg_t              threshold1;
	threshold2_reg_t              threshold2;
	timer_reg_t                   timer;
	reset_reg_t                   reset;
	timer2_reg_t                  timer2;
} device_reg_t;

struct fusb251 {
	struct i2c_client *i2c;
	struct mutex fusb251_irq_mutex;
	struct mutex fusb251_i2c_mutex;
	struct device *dev;
	int irq;
	int moisture_en;
	struct semaphore suspend_lock;
	device_reg_t registers;
	struct switch_dev st_switch;
	struct workqueue_struct *fusb251_wq;
	struct delayed_work en_mos_det_work;
	struct work_struct eint_work;
};

/* Resistance table for moisture detection */
int mos_det_R_table[] = {17, 36, 56, 80, 107, 137, 172, 213,
	262, 320, 391, 480, 594, 747, 960, 1280};

#define MAX_RESISTANCE          1280

#endif